# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "dxi"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||LIS||P|1|20130715141912
    P|1|036666666|||NYME^ANO|NJF|20110513|F|||||PRESCRIBER
    O|1|123456||^^^A|A||||||A||||Serum
    O|2|123456||^^^B|A||||||A||||Serum
    O|3|123456||^^^C|A||||||C||||Serum
    L|1|F
    """
    assert message == expected


def test_decode_query(self):
    worklist = self.decode(
        """
    H|\\^&|||ACCESS^500001|||||LIS||P|1|20141010085833
    Q|1|^12345||ALL||||||||O
    L|1|F
    """
    )
    expected = {
        "context": "query",
        "samples": [{"type": "patient", "sample_id": "12345"}],
    }
    assert worklist == expected


def test_decode_result_status_x(self):
    self.configure(dxi_result_status_x=True)
    worklist = self.decode(
        """
    H|\\^&|||ACCESS^609516|||||LIS||P|1|20230126124530
    P|1|1057729
    O|1|303211010403|^111^4|^^^PSA-Hyb^1|||||||||||Serum||||||||||X
    R|1|^^^PSA-Hyb^1||||||X||||20230126124548|609516|9781^2^272186^4490^^
    C|1|I|IND|I
    L|1|F
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "303211010403",
                "rack": "111-4",
                "results": [
                    {
                        "code": "PSA-Hyb",
                        "comments": [],
                        "date": datetime.datetime(2023, 1, 26, 12, 45, 48),
                        "flags": ["IND"],
                        "value": "X",
                    }
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||ACCESS^500001|||||LIS||P|1|20141010131522
    P|1|Anonyme
    O|1|123456|^9^1|^^^TSH^1|||||||||||Serum||||||||||F
    R|1|^^^TSH^1|0.18|uIU/mL||N||F||||20141010113536
    C|1|I|PEX|I
    R|2|^^^CMV^1|3.33^Negative|ug/dL||N||F||||20141010114333|604718|16344^3^594194^2892
    L|1|F
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "123456",
                "rack": "9-1",
                "results": [
                    {
                        "code": "TSH",
                        "comments": [],
                        "date": datetime.datetime(2014, 10, 10, 11, 35, 36),
                        "flags": ["N", "PEX"],
                        "value": "0.18",
                    },
                    {
                        "code": "CMV",
                        "comments": [],
                        "date": datetime.datetime(2014, 10, 10, 11, 43, 33),
                        "flags": ["N"],
                        "value": "3.33",
                        "lots": [{"lot": "594194-2892", "type": "reagent"}],
                    },
                    {
                        "code": "CMVI",
                        "comments": [],
                        "date": datetime.datetime(2014, 10, 10, 11, 43, 33),
                        "flags": ["N"],
                        "value": "Negative",
                        "lots": [{"lot": "594194-2892", "type": "reagent"}],
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|||ACCESS^604873|||||LIS||P|1|20151209160445
    P|1|127313
    O|1|CQDXI|^4^1|^^^TSH^1|||||||||||Serum||||||||||F
    R|1|^^^TSH^1|14.20|uIU/mL||N||F||||20151209160503|604873
    L|1|F
    """
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "control_lot": "CQDXI",
                "sample_id": "CQDXI",
                "rack": "4-1",
                "results": [
                    {
                        "code": "TSH",
                        "comments": [],
                        "date": datetime.datetime(2015, 12, 9, 16, 5, 3),
                        "flags": ["N"],
                        "value": "14.20",
                    }
                ],
            }
        ],
    }
    assert worklist == expected
