# -*- coding: utf8 -*-

from __future__ import unicode_literals

import datetime

from driverng import Worklist

parser_class = "lxl"


def test_query(self):
    result = self.decode(
        """
    H|\\^&|||lxl01039|||||MCA||P|1
    Q|1|SAMPLE1||ALL||||||||O
    Q|2|SAMPLE2||ALL||||||||O
    Q|3|SAMPLE3||ALL||||||||O
    Q|4|SAMPLE4||ALL||||||||O
    L|1|N"""
    )
    expected = {
        "context": "query",
        "samples": [
            {"sample_id": "SAMPLE1", "type": "patient"},
            {"sample_id": "SAMPLE2", "type": "patient"},
            {"sample_id": "SAMPLE3", "type": "patient"},
            {"sample_id": "SAMPLE4", "type": "patient"},
        ],
    }
    assert result == expected


def test_void_order(self):
    result = self.order(
        worklist=Worklist(context="order"),
        context={
            "query": self.parse(
                """
    H|\\^&|||lxl01039|||||MCA||P|1
    Q|1|SAMPLE1||ALL||||||||O
    L|1|N"""
            )
        },
    )
    expected = """
    H|\\^&|||||||||||1|20130715141912
    L|1|I"""
    assert result == expected


def test_order(self):
    result = self.order()
    expected = """H|\\^&|||||||||||1|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F|||||PRESCRIBER
        O|1|123456||^^^A^1:10\\^^^B|N|20130603110342|||||N||||||||||||||O
        O|2|123456||^^^C|N|20130603110342|||||C||||||||||||||O
        L|1|N"""
    assert result == expected


def test_order_receiver_id(self):
    self.configure(receiver_id="lxl0123456")
    result = self.order(worklist=self.order_worklist)
    expected = """H|\\^&||||||||lxl0123456|||1|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F|||||PRESCRIBER
        O|1|123456||^^^A^1:10\\^^^B|N|20130603110342|||||N||||||||||||||O
        O|2|123456||^^^C|N|20130603110342|||||C||||||||||||||O
        L|1|N"""
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||lxl01039|||||MCA||P|1
    P|1|01130213121|||TEST^ANONYME||19510115|M
    O|1|05124388||^^^25OHD^|R||||||||||||||||||||F
    R|1|^^^25OHD^^DOSE|18.2|Âµg/L||N||F||||20130701122630
    C|1|I|Commentaire|I
    M|1|5228|20131224|129482|00024|3^5
    M|2|5228|20131224|129482|00025|3^5
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "05124388",
                "type": "patient",
                "rack": "3-5",
                "results": [
                    {
                        "flags": ["N", "Commentaire"],
                        "date": datetime.datetime(2013, 7, 1, 12, 26, 30),
                        "lots": [
                            {
                                "expiration": datetime.datetime(2013, 12, 24, 0, 0),
                                "lot": "129482",
                                "type": "reagent",
                            }
                        ],
                        "value": "18.2",
                        "code": "25OHDDOSE",
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc(self):
    result = self.decode(
        """
    H|\\^&|||lxl01111|||||ol2neo||P|1|
    P|1||||^||||||||
    O|1|#6860340||^^^HSV-M^|R||||||Q||||||||||||||F
    R|1|^^^HSV-M^^DOSE|2.76|Index||N||F||||20131023100504
    R|2|^^^HSV-M^^INTR||||N||F||||20131023100504
    M|1|4963|20140513|77051|00039|5^4
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "6860340",
                "control_lot": "6860340",
                "type": "control",
                "results": [
                    {
                        "flags": ["N"],
                        "lots": [],
                        "date": datetime.datetime(2013, 10, 23, 10, 5, 4),
                        "value": "2.76",
                        "code": "HSV-MDOSE",
                    }
                ],
            }
        ],
    }
    assert result == expected
