# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "novodiag"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|@^&|||^Host|||||Novodiag||P|1394-97|20130715141912
    P|1|||036666666|NYME^ANO
    O|1|123456||^^^A|S|20130603110342
    O|2|123456||^^^B|S|20130603110342
    L|1|F"""
    assert message == expected


def test_decode_result(self):
    date = datetime.datetime(2019, 3, 1, 1, 15)
    worklist = self.decode(
        """
    H|@^\\|||^NOVODIAG^TSC-XPS1^123456789001^100000|||||Host||P|LIS02-A2|201903010120
    P|1
    O|1|10000938388||^^^BG|R|000101011200
    R|1|^^^BG^BacterialGE+^2.0.1^18PE1095|POSITIVE|||||F||Administrator|201903010112|201903010115|100000
    R|2|^^^CCOLI^^^Campylobacter coli|NEGATIVE
    R|3|^^^CJEJUNI^^^Campylobacter jejuni|NEGATIVE
    R|4|^^^CDIFFB^^^Clostridium difficile toxin B|NEGATIVE
    R|5|^^^EAEC^^^EAEC|NEGATIVE
    R|6|^^^EHEC4^^^EHEC (eae, stx1)|POSITIVE
    C|1|I|EPEC CANNOT BE RULED OUT|P
    R|7|^^^EPEC^^^EPEC (eae)|N/A
    R|8|^^^ETEC^^^ETEC|NEGATIVE
    R|9|^^^SALMSPP^^^Salmonella spp.|NEGATIVE
    R|10|^^^SHISPP^^^Shigella spp./EIEC|NEGATIVE
    R|11|^^^VCHOL^^^Vibrio cholerae|NEGATIVE
    R|12|^^^VPARA^^^Vibrio parahaemolyticus|NEGATIVE
    R|13|^^^YENTERO^^^Yersinia enterocolitica|NEGATIVE
    R|14|^^^YPSEUDOT^^^Yersinia pseudotuberculosis/pestis|NEGATIVE
    L|1|F"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "10000938388",
                "type": "patient",
                "results": [
                    {
                        "code": "BG",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "POSITIVE",
                    },
                    {
                        "code": "CCOLI",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "CJEJUNI",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "CDIFFB",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "EAEC",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "EHEC4",
                        "comments": ["EPEC CANNOT BE RULED OUT"],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "POSITIVE",
                    },
                    {
                        "code": "EPEC",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "N/A",
                    },
                    {
                        "code": "ETEC",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "SALMSPP",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "SHISPP",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "VCHOL",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "VPARA",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "YENTERO",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                    {
                        "code": "YPSEUDOT",
                        "comments": [],
                        "date": date,
                        "instrument": "TSC-XPS1",
                        "operator": "Administrator",
                        "value": "NEGATIVE",
                    },
                ],
            }
        ],
    }
    assert worklist == expected
