# -*- coding: utf8 -*-
from __future__ import unicode_literals

import copy
import datetime

from driverng.tests import BaseTest


class TestRapidlab(BaseTest):
    MAPPING_CLASS = "rapidlab"

    def assertMessageEqual(self, result, expected):
        mresult = self.F.parse(result)
        mexpected = self.F.parse(expected)
        rkeys = sorted(mresult.fields.keys())
        ekeys = sorted(mexpected.fields.keys())
        self.assertEqual(rkeys, ekeys)
        for key, rfield in mresult.fields.items():
            efield = mexpected.fields[key]
            for attr in ("prefix", "name", "value", "units", "exc_group"):
                rvalue = getattr(rfield, attr)
                evalue = getattr(efield, attr)
                self.assertEqual(
                    rvalue,
                    evalue,
                    "key '%s' unexpected value '%s' != '%s" % (key, rvalue, evalue),
                )

    def test_order(self):
        self.assertOrderMessageEqual(
            self.WORKLIST_SIMPLE1,
            "PAT_DEMOG_DATA\x1c\x1eiDOB\x1d13May2011\x1d\x1d\x1d\x1ciPID\x1d123456\x1d\x1d\x1d\x1ciSEX\x1dF\x1d\x1d\x1d\x1ciLNAME\x1dNYME\x1d\x1d\x1d\x1ciFNAME\x1dANO\x1d\x1d\x1d\x1ciTIME\x1d11:42:42\x1d\x1d\x1d\x1ciDATE\x1d13May2013\x1d\x1d\x1d\x1c\x1e",
        )

    def test_order_nodob(self):
        worklist = copy.deepcopy(self.WORKLIST_SIMPLE1)
        worklist.samples[0]["birthdate"] = ""
        self.assertOrderMessageEqual(
            worklist,
            "PAT_DEMOG_DATA\x1c\x1eiDOB\x1d\x1d\x1d\x1d\x1ciPID\x1d123456\x1d\x1d\x1d\x1ciSEX\x1dF\x1d\x1d\x1d\x1ciLNAME\x1dNYME\x1d\x1d\x1d\x1ciFNAME\x1dANO\x1d\x1d\x1d\x1ciTIME\x1d11:42:42\x1d\x1d\x1d\x1ciDATE\x1d13May2013\x1d\x1d\x1d\x1c\x1e",
        )

    def test_void_order(self):
        self.assertVoidOrderEqual(
            """PAT_DEMOG_REQ\x1c\x1eaMOD\x1d1265\x1d\x1d\x1d\x1ciIID\x1d18131\x1d\x1d\x1d\x1ciPID\x1d0037060600822\x1d\x1d\x1d\x1c\x1e""",
            """PAT_DEMOG_NOT_AV\x1c\x1eaMOD\x1d1265\x1d\x1d\x1d\x1ciIID\x1d18131\x1d\x1d\x1d\x1ciPID\x1d0037060600822\x1d\x1d\x1d\x1c\x1e""",
        )

    def test_query(self):
        self.assertQueryEqual(
            """PAT_DEMOG_REQ\x1c\x1eaMOD\x1d1265\x1d\x1d\x1d\x1ciIID\x1d18131\x1d\x1d\x1d\x1ciPID\x1d0037060600822\x1d\x1d\x1d\x1c\x1e
            """,
            ["0037060600822"],
        )

    def test_result(self):
        date = datetime.datetime(2013, 4, 11, 9, 35)
        self.assertResultEqual(
            "SMP_NEW_DATA\x1c\x1eaMOD\x1d1265\x1d\x1d\x1d\x1ciIID\x1d16080\x1d\x1d\x1d\x1crDATE\x1d11Apr2013\x1d\x1d\x1d\x1crDEVICE\x1dSYRINGE\x1d\x1d\x1d\x1crTIME\x1d09:35:52\x1d\x1d\x1d\x1crTYPE\x1dSAMPLE\x1d\x1d\x1d\x1ciSOURCE\x1dARTERIAL\x1d\x1d\x1d\x1crSEQ\x1d469031\x1d\x1d\x1d\x1crRCartID\x1d2305703701\x1d\x1d\x1d\x1crWCartID\x1d304344504\x1d\x1d\x1d\x1cmpH\x1d7.446\x1d \x1dBUB\x1d\x1cmPCO2\x1d5.70\x1dkPa\x1d\x1d\x1cmPO2\x1d5.96\x1dkPa\x1d\x1d\x1cmNa+\x1d137.7\x1dmmol/L\x1d\x1d\x1cmK+\x1d3.74\x1dmmol/L\x1d\x1d\x1cmCa++\x1d1.28\x1dmmol/L\x1d\x1d\x1cmLactate\x1d1.75\x1dmmol/L\x1d\x1d\x1cmtHb\x1d110\x1dg/L\x1d\x1d\x1cmO2Hb\x1d0.822\x1d \x1d\x1d\x1cmCOHb\x1d0.008\x1d \x1d\x1d\x1cmMetHb\x1d0.002\x1d \x1d\x1d\x1cmHHb\x1d0.168\x1d \x1d\x1d\x1cmBP\x1d738\x1dmmHg\x1d\x1d\x1ciPID\x1d0031685994\x1d\x1d\x1d\x1ccHCO3act\x1d28.8\x1dmmol/L\x1d\x1d\x1ccBE(vt)\x1d4.3\x1dmmol/L\x1d\x1d\x1cctCO2\x1d30.1\x1dmmol/L\x1d\x1d\x1ccctO2(a)\x1d12.7\x1dmL/dL\x1d\x1d\x1cmsO2\x1d0.830\x1d \x1d\x1d\x1ccP50\x1d3.46\x1dkPa\x1d\x1d\x1c\x1e",
            {
                "0031685994": {
                    "resultats": [
                        {"date": date, "resu": "3.74", "test": "K+"},
                        {"date": date, "resu": "738", "test": "BP"},
                        {
                            "date": date,
                            "resu": "7.446",
                            "test": "pH",
                            "alarmes": ["BUB"],
                        },
                        {"date": date, "resu": "0.008", "test": "COHb"},
                        {"date": date, "resu": "ARTERIAL", "test": "SOURCE"},
                        {"date": date, "resu": "0.830", "test": "sO2"},
                        {"date": date, "resu": "12.7", "test": "ctO2(a)"},
                        {"date": date, "resu": "28.8", "test": "HCO3act"},
                        {"date": date, "resu": "3.46", "test": "P50"},
                        {"date": date, "resu": "30.1", "test": "tCO2"},
                        {"date": date, "resu": "137.7", "test": "Na+"},
                        {"date": date, "resu": "5.96", "test": "PO2"},
                        {"date": date, "resu": "4.3", "test": "BE(vt)"},
                        {"date": date, "resu": "0.168", "test": "HHb"},
                        {"date": date, "resu": "1.28", "test": "Ca++"},
                        {"date": date, "resu": "1.75", "test": "Lactate"},
                        {"date": date, "resu": "0.822", "test": "O2Hb"},
                        {"date": date, "resu": "110", "test": "tHb"},
                        {"date": date, "resu": "5.70", "test": "PCO2"},
                        {"date": date, "resu": "0.002", "test": "MetHb"},
                    ]
                }
            },
        )

    def test_patient(self):
        date = datetime.datetime(2013, 4, 11, 6, 1)
        lot = "AQC12"
        self.assertQCEqual(
            "QC_NEW_DATA\x1c\x1eaMOD\x1d1265\x1d\x1d\x1d\x1ciIID\x1d16080\x1d\x1d\x1d\x1crDATE\x1d11Apr2013\x1d\x1d\x1d\x1crDEVICE\x1dAQC\x1d\x1d\x1d\x1crTIME\x1d06:01:41\x1d\x1d\x1d\x1crTYPE\x1dQC\x1d\x1d\x1d\x1crSEQ\x1d468908\x1d\x1d\x1d\x1crRCartID\x1d2305703701\x1d\x1d\x1d\x1crWCartID\x1d304344504\x1d\x1d\x1d\x1ciQLEV\x1d2\x1d\x1d\x1d\x1ciQID\x1dAQC1\x1d\x1d\x1d\x1crAQCID\x1d1361536209\x1d\x1d\x1d\x1ciSTATUS\x1dACCEPTED\x1d\x1d\x1d\x1csLQmpH\x1d7.330\x1d \x1d\x1d\x1cmpH\x1d7.347\x1d \x1d\x1d\x1csHQmpH\x1d7.370\x1d \x1d\x1d\x1csLQmPCO2\x1d4.67\x1dkPa\x1d\x1d\x1cmPCO2\x1d5.35\x1dkPa\x1d\x1d\x1csHQmPCO2\x1d6.00\x1dkPa\x1d\x1d\x1csLQmPO2\x1d12.29\x1dkPa\x1d\x1d\x1cmPO2\x1d13.34\x1dkPa\x1d\x1d\x1csHQmPO2\x1d14.37\x1dkPa\x1d\x1d\x1csLQmNa+\x1d130.0\x1dmmol/L\x1d\x1d\x1cmNa+\x1d135.4\x1dmmol/L\x1d\x1d\x1csHQmNa+\x1d140.0\x1dmmol/L\x1d\x1d\x1csLQmK+\x1d4.70\x1dmmol/L\x1d\x1d\x1cmK+\x1d5.00\x1dmmol/L\x1d\x1d\x1csHQmK+\x1d5.30\x1dmmol/L\x1d\x1d\x1csLQmCa++\x1d1.10\x1dmmol/L\x1d\x1d\x1cmCa++\x1d1.18\x1dmmol/L\x1d\x1d\x1csHQmCa++\x1d1.30\x1dmmol/L\x1d\x1d\x1csLQmLactate\x1d0.66\x1dmmol/L\x1d\x1d\x1cmLactate\x1d0.85\x1dmmol/L\x1d\x1d\x1csHQmLactate\x1d1.14\x1dmmol/L\x1d\x1d\x1csLQmtHb\x1d126\x1dg/L\x1d\x1d\x1cmtHb\x1d142\x1dg/L\x1d\x1d\x1csHQmtHb\x1d154\x1dg/L\x1d\x1d\x1csLQmO2Hb\x1d0.890\x1d \x1d\x1d\x1cmO2Hb\x1d0.927\x1d \x1d\x1d\x1csHQmO2Hb\x1d0.950\x1d \x1d\x1d\x1csLQmCOHb\x1d-0.015\x1d \x1d\x1d\x1cmCOHb\x1d0.021\x1d \x1d\x1d\x1csHQmCOHb\x1d0.085\x1d \x1d\x1d\x1csLQmMetHb\x1d-0.026\x1d \x1d\x1d\x1cmMetHb\x1d0.019\x1d \x1d\x1d\x1csHQmMetHb\x1d0.066\x1d \x1d\x1d\x1csLQmHHb\x1d-0.015\x1d \x1d\x1d\x1cmHHb\x1d0.033\x1d \x1d\x1d\x1csHQmHHb\x1d0.065\x1d \x1d\x1d\x1cmBP\x1d738\x1dmmHg\x1d\x1d\x1c\x1e",
            [
                {"date": date, "lot": lot, "resu": "4.67", "test": "LQmPCO2"},
                {"date": date, "lot": lot, "resu": "140.0", "test": "HQmNa+"},
                {"date": date, "lot": lot, "resu": "1.18", "test": "Ca++"},
                {"date": date, "lot": lot, "resu": "0.065", "test": "HQmHHb"},
                {"date": date, "lot": lot, "resu": "0.085", "test": "HQmCOHb"},
                {"date": date, "lot": lot, "resu": "5.00", "test": "K+"},
                {"date": date, "lot": lot, "resu": "0.890", "test": "LQmO2Hb"},
                {"date": date, "lot": lot, "resu": "-0.015", "test": "LQmHHb"},
                {"date": date, "lot": lot, "resu": "738", "test": "BP"},
                {"date": date, "lot": lot, "resu": "7.347", "test": "pH"},
                {"date": date, "lot": lot, "resu": "12.29", "test": "LQmPO2"},
                {"date": date, "lot": lot, "resu": "6.00", "test": "HQmPCO2"},
                {"date": date, "lot": lot, "resu": "0.021", "test": "COHb"},
                {"date": date, "lot": lot, "resu": "-0.015", "test": "LQmCOHb"},
                {"date": date, "lot": lot, "resu": "126", "test": "LQmtHb"},
                {"date": date, "lot": lot, "resu": "7.370", "test": "HQmpH"},
                {"date": date, "lot": lot, "resu": "1.14", "test": "HQmLactate"},
                {"date": date, "lot": lot, "resu": "0.66", "test": "LQmLactate"},
                {"date": date, "lot": lot, "resu": "154", "test": "HQmtHb"},
                {"date": date, "lot": lot, "resu": "14.37", "test": "HQmPO2"},
                {"date": date, "lot": lot, "resu": "1.10", "test": "LQmCa++"},
                {"date": date, "lot": lot, "resu": "135.4", "test": "Na+"},
                {"date": date, "lot": lot, "resu": "13.34", "test": "PO2"},
                {"date": date, "lot": lot, "resu": "0.033", "test": "HHb"},
                {"date": date, "lot": lot, "resu": "5.30", "test": "HQmK+"},
                {"date": date, "lot": lot, "resu": "130.0", "test": "LQmNa+"},
                {"date": date, "lot": lot, "resu": "0.85", "test": "Lactate"},
                {"date": date, "lot": lot, "resu": "0.927", "test": "O2Hb"},
                {"date": date, "lot": lot, "resu": "-0.026", "test": "LQmMetHb"},
                {"date": date, "lot": lot, "resu": "142", "test": "tHb"},
                {"date": date, "lot": lot, "resu": "0.950", "test": "HQmO2Hb"},
                {"date": date, "lot": lot, "resu": "1.30", "test": "HQmCa++"},
                {"date": date, "lot": lot, "resu": "0.066", "test": "HQmMetHb"},
                {"date": date, "lot": lot, "resu": "5.35", "test": "PCO2"},
                {"date": date, "lot": lot, "resu": "4.70", "test": "LQmK+"},
                {"date": date, "lot": lot, "resu": "0.019", "test": "MetHb"},
                {"date": date, "lot": lot, "resu": "7.330", "test": "LQmpH"},
            ],
        )


class TestProtocolRapidlab(BaseTest):
    PROTOCOL_CLASS = "rapidlab"

    def test_send(self):
        self.assertSendMessageEqual(
            """PAT_DEMOG_DATA\x1c\x1eiDOB\x1d13May2011\x1d\x1d\x1d\x1ciPID\x1d123456\x1d\x1d\x1d\x1ciSEX\x1dF\x1d\x1d\x1d\x1ciLNAME\x1dNYME\x1d\x1d\x1d\x1ciFNAME\x1dANO\x1d\x1d\x1d\x1ciTIME\x1d11:42:42\x1d\x1d\x1d\x1ciDATE\x1d13May2013\x1d\x1d\x1d\x1c\x1e""",
            """
            send <stx>PAT_DEMOG_DATA<fs><rs>iDOB<gs>13May2011<gs><gs><gs><fs>iPID<gs>123456<gs><gs><gs><fs>iSEX<gs>F<gs><gs><gs><fs>iLNAME<gs>NYME<gs><gs><gs><fs>iFNAME<gs>ANO<gs><gs><gs><fs>iTIME<gs>11:42:42<gs><gs><gs><fs>iDATE<gs>13May2013<gs><gs><gs><fs><rs><etx>EC<eot>
            recv <stx>
            recv <ack><etx>0B<eot>
            """,
        )
