# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "remisol"


def test_encode_order(self):
    self.configure(option_envoi_commentaire_patient=True)
    message = self.order()
    expected = """
    H|^~\\&|||MCA||ORM|||REMISOL||P|H2.1^L|20130715141912
    P|1|036666666|||NYME^ANO|NJF|20110513|F|||||||||||||Commentaire patient no1|Commentaire patient no2
    C|1||Patient commentaire no1
    C|2||Patient commentaire no2
    OBR|1|123456^78910||A|A||20130513114242||||N|||||^PRESCRIBER^L^^SER^L
    OBX|1|NM|A||42
    OBR|2|123456^78910||B|A||20130513114242||||A|||||^PRESCRIBER^L^^SER^L
    OBX|1|NM|B||201302131516|||||||||20110506121300
    OBR|3|123456^78910||C|A||20130513114242||||C|||||^PRESCRIBER^L^^SER^L
    L|1||1|10
    """
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|^~\\&|||REMISOL^REMISOL||ORU|||LIS^LIS|UPLOAD_SIGN=1\\INTERPRETATION_FLAG=0\\INSTRUMENT_FLAG=0|P|H2.2^L|20140924094023
    P|1|010626262465|||TEST^Anonyme||19791115|M|||||||||||||||||^^A2
    OBR|1|82401701^557312||NA^Sodium se^L~K^Potassium se^L~CL^Chlore se^L~UREE^Uree serum^L~CREA^Creatinine^L~GLU^Glycemie^L|R||20140924082033||||R|||20140924082549||12504^VANPEC^L^12486^A2^L||||||20140924094023
    OBX|1|NM|NA^Sodium se^L||140.0|mmol/L||N|||||AU_480
    OBX|2|NM|K^Potassium se^L||4.31|mmol/L||N|||||AU_480
    OBX|3|NM|CL^Chlore se^L||103.0|mmol/L||N|||||AU_480
    OBX|4|NM|UREE^Uree serum^L|5460^7664|0.17|g/L||N|||||AU_480
    OBX|5|NM|CREA^Creatinine^L|6152^L626|8.60|mg/L||N|||||AU_480
    C|1|M|Resultat Controle
    C|2|M|2e commentaire
    OBX|6|NM|GLU^Glycemie^L|4936^7036|1.16|g/L||H|||||AU_480
    L|1||1|11
    """
    )
    date = datetime.datetime(2014, 9, 24, 9, 40, 23)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "NA",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "AU_480",
                        "value": "140.0",
                    },
                    {
                        "code": "K",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "AU_480",
                        "value": "4.31",
                    },
                    {
                        "code": "CL",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "AU_480",
                        "value": "103.0",
                    },
                    {
                        "code": "UREE",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "AU_480",
                        "value": "0.17",
                    },
                    {
                        "code": "CREA",
                        "comments": ["Resultat Controle", "2e commentaire"],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "AU_480",
                        "value": "8.60",
                    },
                    {
                        "code": "GLU",
                        "comments": [],
                        "date": date,
                        "flags": ["H"],
                        "instrument": "AU_480",
                        "value": "1.16",
                    },
                ],
                "sample_id": "82401701",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_result_comment(self):
    worklist = self.decode(
        """
    H|^~\\&|||REMISOL^REMISOL||ORU|||REMISOL^REMISOL|UPLOAD_SIGN=1\\INTERPRETATION_FLAG=0\\INSTRUMENT_FLAG=0|P|H2.1^L|20210610162620
    P|1|464203|||TEST HEMATO^COMMENTAIRE||19920610|M|||||||||||||||||^^AB_GRATU
    OBR|1|011061008263^C1061080263||RBC^Globules Rouges^L~HGB^Hemoglobine^L~HCT^Hematocrite^L~MCV^VMC^L~MCH^TCMH^L~MCHC^CCMH^L~PLT^Plaquettes^L~MPV^VMP^L~WBC^Globules Blancs^L~NE^NE%^L~EO^EO%^L~BA^BA%^L~LY^LY%^L~MO^MO%^L~NE#
    A|^NE#^L~EO#^EO#^L~BA#^BA#^L~LY#^LY#^L~MO#^MO#^L|R||20210610161000||||R|||20210610160134||73193^AB_GARIS^L^73192^AB_GRATU^L
    C|1|M|706_Presence de macro-thrombocytes
    C|2|M|617_Anemie microcytaire
    C|3|M|800_Erythromyelemie
    C|4|M|900_Formule sanguine effectuee en cytofluorometrie (HematoFlow Beckman-Coulter)
    OBX|1|NM|RBC^Globules Rouges^L||2.59|T/L||L|||||BD45633|||CFH
    L|1||1
    """
    )
    date = datetime.datetime(2021, 6, 10, 16, 26, 20)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "RBC",
                        "comments": [],
                        "date": date,
                        "flags": ["L"],
                        "instrument": "BD45633",
                        "value": "2.59",
                    },
                    {
                        "code": "COM1",
                        "date": date,
                        "value": "706_Presence de macro-thrombocytes",
                    },
                    {"code": "COM2", "date": date, "value": "617_Anemie microcytaire"},
                    {"code": "COM3", "date": date, "value": "800_Erythromyelemie"},
                    {
                        "code": "COM4",
                        "date": date,
                        "value": "900_Formule sanguine effectuee en cytofluorometrie (HematoFlow Beckman-Coulter)",
                    },
                ],
                "sample_id": "011061008263",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected


def test_decode_interpretation_flags(self):
    worklist = self.decode(
        """
    H|^~\\&|||REMISOL^REMISOL||ORU|||REMISOL^REMISOL|UPLOAD_SIGN=1\\INTERPRETATION_FLAG=1\\INSTRUMENT_FLAG=1|P|H2.1^L|20230313164353
    P|1|200610000233|||TEST^Plaq < 3||20000101|M||
    OBR|1|153031325025^3031325025||PLT^Plaquettes^L|R||20230313164200||||R|||20230313164332|
    OBX|1|TX|PLT^Plaquettes^L||1^<3 N|G/L||N|||||cco|||cco
    L|1
    """
    )
    date = datetime.datetime(2023, 3, 13, 16, 43, 53)
    expected = {
        "context": "result",
        "samples": [
            {
                "results": [
                    {
                        "code": "PLT",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "cco",
                        "value": "1",
                    },
                    {
                        "code": "PLT_FLAG",
                        "comments": [],
                        "date": date,
                        "flags": ["N"],
                        "instrument": "cco",
                        "value": "<3;N",
                    },
                ],
                "sample_id": "153031325025",
                "type": "patient",
            }
        ],
    }
    assert worklist == expected
