# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "variant"


def test_encode_order(self):
    message = self.order()
    expected = """
    H|\\^&||||||||||P||20130715141912
    P|1|036666666|||NYME^ANO||20110513|F|||||PRESCRIBER
    O|1|123456||^^^A
    O|2|123456||^^^B
    L|1|N
    """
    assert message == expected


def test_decode_result(self):
    worklist = self.decode(
        """
    H|\\^&|||Bio-Rad CDM System^CDM 5.1 VII TURBO Instrument 11817|||||||P||20130717124720
    P|1|03022636|||TEST^ANONYME||19780924|M
    O|1|600717007255^009^01||^^^4||||||||||||||||||||1
    C|1|I|D-100 Format|I
    R|1|^^^A1a^AREA|0.7||||||||||1
    R|2|^^^A1a^TIME|0.161||||||||||1
    R|3|^^^A1b^AREA|1.4||||||||||1
    R|4|^^^A1b^TIME|0.232||||||||||1
    R|5|^^^F^AREA|0.8||||||||||1
    R|6|^^^F^TIME|0.293||||||||||1
    R|7|^^^LA1c^AREA|2.2||||||||||1
    R|8|^^^LA1c^TIME|0.412||||||||||1
    R|9|^^^A1c^AREA|7.1||||||||||1
    R|10|^^^A1cIFCC^AREA|54||||||||||1
    R|11|^^^A1c^TIME|0.517||||||||||1
    R|12|^^^P3^AREA|3.9||||||||||1
    R|13|^^^P3^TIME|0.769||||||||||1
    R|14|^^^P4^AREA|1.4||||||||||1
    R|15|^^^P4^TIME|0.865||||||||||1
    R|16|^^^Ao^AREA|83.1||||||||||1
    R|17|^^^Ao^TIME|1.019||||||||||1
    R|18|^^^TOTAL^AREA|1.39||||||||||1
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "patient",
                "sample_id": "600717007255",
                "rack": "009-01",
                "results": [
                    {
                        "code": "COMMENT_4_1",
                        "date": datetime.datetime(2013, 7, 17, 12, 47, 20),
                        "value": "D-100 Format",
                    },
                    {
                        "code": "A1aAREA",
                        "date": None,
                        "value": "0.7",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1aTIME",
                        "date": None,
                        "value": "0.161",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1bAREA",
                        "date": None,
                        "value": "1.4",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1bTIME",
                        "date": None,
                        "value": "0.232",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "FAREA",
                        "date": None,
                        "value": "0.8",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "FTIME",
                        "date": None,
                        "value": "0.293",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "LA1cAREA",
                        "date": None,
                        "value": "2.2",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "LA1cTIME",
                        "date": None,
                        "value": "0.412",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1cAREA",
                        "date": None,
                        "value": "7.1",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1cIFCCAREA",
                        "date": None,
                        "value": "54",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1cTIME",
                        "date": None,
                        "value": "0.517",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "P3AREA",
                        "date": None,
                        "value": "3.9",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "P3TIME",
                        "date": None,
                        "value": "0.769",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "P4AREA",
                        "date": None,
                        "value": "1.4",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "P4TIME",
                        "date": None,
                        "value": "0.865",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "AoAREA",
                        "date": None,
                        "value": "83.1",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "AoTIME",
                        "date": None,
                        "value": "1.019",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "TOTALAREA",
                        "date": None,
                        "value": "1.39",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                ],
            }
        ],
    }
    assert worklist == expected


def test_decode_control(self):
    worklist = self.decode(
        """
    H|\\^&|||Bio-Rad CDM System^CDM 5.1 VII TURBO Instrument 11817|||||||P||20130717114851
    P|1
    O|1|HC-1-33832^003^01||^^^4||||||||||||||||||||1
    R|1|^^^A1a^AREA|1.0||||||||20140227160959||1
    R|2|^^^A1a^TIME|0.162||||||||||1
    R|3|^^^A1b^AREA|1.3||||||||||1
    R|4|^^^A1b^TIME|0.229||||||||||1
    R|5|^^^F^AREA|1.1||||||||||1
    R|6|^^^F^TIME|0.290||||||||||1
    R|7|^^^LA1c^AREA|1.2||||||||||1
    R|8|^^^LA1c^TIME|0.418||||||||||1
    R|9|^^^A1c^AREA|9.8||||||||||1
    R|10|^^^A1cIFCC^AREA|84||||||||||1
    R|11|^^^A1c^TIME|0.509||||||||||1
    R|12|^^^P3^AREA|3.2||||||||||1
    R|13|^^^P3^TIME|0.770||||||||||1
    R|14|^^^P4^AREA|1.7||||||||||1
    R|15|^^^P4^TIME|0.859||||||||||1
    R|16|^^^Ao^AREA|82.1||||||||||1
    R|17|^^^Ao^TIME|1.014||||||||||1
    R|18|^^^TOTAL^AREA|1.38||||||||||1
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "type": "control",
                "sample_id": "HC-1-33832",
                "control_lot": "HC-1-33832",
                "rack": "003-01",
                "results": [
                    {
                        "code": "A1aAREA",
                        "date": datetime.datetime(2014, 2, 27, 16, 9, 59),
                        "value": "1.0",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1aTIME",
                        "date": None,
                        "value": "0.162",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1bAREA",
                        "date": None,
                        "value": "1.3",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1bTIME",
                        "date": None,
                        "value": "0.229",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "FAREA",
                        "date": None,
                        "value": "1.1",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "FTIME",
                        "date": None,
                        "value": "0.290",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "LA1cAREA",
                        "date": None,
                        "value": "1.2",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "LA1cTIME",
                        "date": None,
                        "value": "0.418",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1cAREA",
                        "date": None,
                        "value": "9.8",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1cIFCCAREA",
                        "date": None,
                        "value": "84",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "A1cTIME",
                        "date": None,
                        "value": "0.509",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "P3AREA",
                        "date": None,
                        "value": "3.2",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "P3TIME",
                        "date": None,
                        "value": "0.770",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "P4AREA",
                        "date": None,
                        "value": "1.7",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "P4TIME",
                        "date": None,
                        "value": "0.859",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "AoAREA",
                        "date": None,
                        "value": "82.1",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "AoTIME",
                        "date": None,
                        "value": "1.014",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                    {
                        "code": "TOTALAREA",
                        "date": None,
                        "value": "1.38",
                        "instrument": "CDM 5.1 VII TURBO Instrument 11817",
                    },
                ],
            }
        ],
    }

    assert worklist == expected
