# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime

parser_class = "vitros"


def test_order(self):
    result = self.order()
    expected = """
        H|\\^&|||MCA||||||||LIS2-A|20130715141912
        P|1|036666666|||NYME^ANO||20110513|F
        O|1|123456||^^^1.0000+A+1.0\\B+1.0|A||20130513114242||||N||||Serum||||||||||O
        O|2|123456||^^^1.0000+C+1.0|A||20130513114242||||C||||Serum||||||||||O
        L|1"""
    assert result == expected


def test_result(self):
    result = self.decode(
        """
    H|\\^&|||5600780||||||||LIS2-A|20130718144840
    P|1|030221388760|||ANO^NYME||19910217|F
    O|1|3299606440^66^8||^^^1.0000+300+4.0\\950+1.0\\951+1.0\\952+1.0|R||20130718114150||||N||||3||||||||||F
    R|1|^^^1.0000+300+4.0|42.134|mmol/L||^0^M1RD\\^0^NR\\^0^NR\\^0^NR||V||MURIELLE|20130718144131|20130718144840|56000780
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "3299606440",
                "type": "patient",
                "rack": "66-8",
                "results": [
                    {
                        "flags": ["M1", "RD"],
                        "instrument": "56000780",
                        "date": datetime.datetime(2013, 7, 18, 14, 48, 40),
                        "value": "42.134",
                        "code": "300",
                        "operator": "MURIELLE",
                        "lots": [],
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_erreur_vitros_fusion(self):
    result = self.decode(
        """
        H|\\^&|||1284||||||||E 1394-97|20140408124440
        M|1|E|PY1|034|N|L'ID 010104080169 a été trouvée au mauvais emplacement|20140408124440
        L|1|N
        """
    )
    expected = {"context": "result", "samples": []}
    assert result == expected


def test_alarme(self):
    result = self.decode(
        """
    H|\\^&|||1284||||||||E 1394-97|20140408102700
    P|1|05000000021|||ESSAI CHIMIE||19000000|U
    O|1|010104080149^34^1||^^^1.0000+344+1.0|R||20140408112500||||N||||1||||||||||F
    R|1|^^^1.0000+344+1.0|90.0|mg/L||^4^OR\\^J^\\^0^\\^J^||V|||20140408101627|20140408102529|34001284
    L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "010104080149",
                "type": "patient",
                "rack": "34-1",
                "results": [
                    {
                        "flags": ["4", "OR", "HJ", "TJ"],
                        "instrument": "34001284",
                        "date": datetime.datetime(2014, 4, 8, 10, 25, 29),
                        "value": "90.0",
                        "code": "344",
                        "lots": [],
                        "operator": None,
                    }
                ],
            }
        ],
    }
    assert result == expected


def test_qc_result(self):
    result = self.decode(
        """
H|\\^&|||1284||||||||E 1394-97|20140528071221
P|1
O|1|CQURI1^3^3||^^^1.0000+300+1.0\\302+1.0\\308+1.0\\309+1.0\\311+1.0\\314+1.0\\315+1.0\\318+1.0\\348+1.0\\980+1.0|R||20140528052600||||N||||3||||||||||F
R|1|^^^1.0000+980+1.0|7.88|mg/L||^0^NQ\\^0^NR\\^0^NR\\^0^NR||V|||20140528070035|20140528071221|34001284
        M|1|X|12J25/050313^20151031000000^20140514073618^X3482^W3524|20131213105146^N|64341^19740423174056^20150331000000
        L|1|N"""
    )
    expected = {
        "context": "result",
        "samples": [
            {
                "sample_id": "64341",
                "type": "control",
                "control_lot": "64341",
                "results": [
                    {
                        "instrument": "34001284",
                        "date": datetime.datetime(2014, 5, 28, 7, 12, 21),
                        "lots": [
                            {
                                "expiration": datetime.datetime(2015, 10, 31, 0, 0),
                                "lot": "12J25/050313",
                                "type": "control",
                            }
                        ],
                        "value": "7.88",
                        "code": "980",
                        "flags": ["NQ"],
                        "operator": None,
                    }
                ],
            }
        ],
    }
    assert result == expected
